package org.sanskritlibrary.lexicons;

import java.util.Hashtable;
import java.util.ArrayList;
import java.nio.charset.Charset;
import java.io.FileInputStream;
import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathFactory;
import javax.xml.xpath.XPathConstants;
import java.io.OutputStreamWriter;
import java.io.InputStream;
//import org.sanskritlibrary.webservice.TransformMapEntry;

public class FSM
{

	static int processLineNum = 0;
	static Hashtable<String, Hashtable<String, Boolean>> badChars =
			  new Hashtable<String, Hashtable<String, Boolean>>();
	String startState;
	String inputDecoding;
	String outputEncoding;
	Hashtable<String, Hashtable<Character, ArrayList<FSMEntry>>> states =
			  new Hashtable<String, Hashtable<Character, ArrayList<FSMEntry>>>();
	FSMEntry[] fsmEntry;
	Hashtable<Character, Integer> missing = new Hashtable<Character, Integer>();
	
	public FSM(String startState, String inputDecoding, String outputEncoding, FSMEntry[] fsmEntry)
	{
		this.startState = startState;
		this.inputDecoding = inputDecoding;
		this.outputEncoding = outputEncoding;
		this.fsmEntry = fsmEntry;
		int linenum = 0;  // for debugging;
		for (FSMEntry fe : fsmEntry)
		{
			linenum++;
			if (fe == null)
			{
				continue;		// probably an ignored element in a converted XML file
			}
			for (String state : fe.stateName)
			{
				Hashtable<Character, ArrayList<FSMEntry>> entry = null;
				if (!states.containsKey(state))
				{
					entry = new Hashtable<Character, ArrayList<FSMEntry>>();
					states.put(state, entry);
				} else
				{
					entry = states.get(state);
				}

				ArrayList<FSMEntry> list = null;
				if (fe.edge.length() == 0) // should not happen
				{
					continue;
				}
				if (!entry.containsKey(fe.edge.charAt(0)))
				{
					list = new ArrayList<FSMEntry>();
					entry.put(fe.edge.charAt(0), list);
				} else
				{
					list = entry.get(fe.edge.charAt(0));
				}

				list.add(fe);
			}
		}
	}

	public void createInverse(String outputFile) throws Exception
	{
		try
		{
			FileOutputStream fos = new FileOutputStream(outputFile);
			BufferedOutputStream bos = new BufferedOutputStream(fos);
			OutputStreamWriter osw = new OutputStreamWriter(bos, new RawEncoder(Charset.forName("UTF-8"), 1F, 1F));
			BufferedWriter writer = new BufferedWriter(osw);
			writer.write(String.format("<fsm start='%s' inputDecoding='%s' outputEncoding='%s'>\n",
					  startState, outputEncoding, inputDecoding));
			int lineno = 0;
			int bufferSize = 0;
			for (FSMEntry fe : fsmEntry)
			{
				lineno++;
				String element = String.format("<e> <s>%s</s> <in>%s</in> <out>%s</out></e>\n",
						  "INIT", toUnicodeString(fe.target), toUnicodeString(fe.edge));
				writer.write(element);
				/*
				bufferSize += element.length();
				if (bufferSize > 2048)
				{
				writer.flush();
				bufferSize = 0;
				}
				 */
			}
			writer.write("</fsm>");
			//writer.flush();
			writer.close();
		} catch (Exception ex)
		{
			throw ex;
		}
	}

	public String processLine(String line)
	{
		String currentStateKey = startState;
		Hashtable<Character, ArrayList<FSMEntry>> currentState = states.get(currentStateKey);
		processLineNum++;		// for debugging purposes
		if (processLineNum == 6)
		{
			processLineNum = 6;
		}
		StringBuilder result = new StringBuilder();		// to avoid lots of String objects being created.
		int n = 0;
		while (n < line.length())
		{
			boolean processed = false;
			ArrayList<FSMEntry> list = null;
			if (currentState == null)
			{
				currentState = null;
			}
			if (currentState.containsKey(line.charAt(n)))
			{
				FSMEntry bestFE = null;
				String best = "";
				list = currentState.get(line.charAt(n));
				for (FSMEntry fe : list)
				{
					String match = Test(line, n, fe);
					if (match.length() > best.length())
					{
						best = match;
						bestFE = fe;
					}
				}
				if (bestFE != null)
				{
					n += best.length();
					result.append(bestFE.target);
					processed = true;
					if (bestFE.nextState != null && !bestFE.nextState.equals(currentStateKey))
					{
						currentStateKey = bestFE.nextState;
						currentState = states.get(currentStateKey);
					}
				}
			}

			if (!processed)		// send character through as is
			{
				int cnt = 0;
				char ch = line.charAt(n);
				if (missing.get(ch) != null)
					cnt = missing.get(ch);
				cnt++;
				missing.put(ch, cnt);
				result.append(line.charAt(n));
				n++;
			}
		}

		return result.toString();
	}

	public Hashtable<Character, Integer> getMissing()
	{
		return missing;
	}
	
	public String ToXML()
	{
		StringBuilder sb = new StringBuilder();
		sb.append("<fsm start='INIT'>");
		for (FSMEntry fe : fsmEntry)
		{
			String delim = "";
			sb.append("<e> <s>");
			for (String state : fe.stateName)
			{
				sb.append(String.format("%s%s", delim, state));
				delim = ",";
			}

			sb.append("</s>");
			sb.append(String.format(" <in>{0}", toUnicodeString(fe.edge)));
			if (fe.regex != null)
			{
				sb.append(String.format("/%s", fe.regex));
			}
			sb.append("</in>");
			sb.append(String.format(" <out>{0}</out>", toUnicodeString(fe.target)));
			sb.append(String.format(" <next>{0}</next>", fe.nextState));
			sb.append("</e>\n");
		}

		sb.append("</fsm>");
		return sb.toString();
	}

	public static String toUnicodeString(String s)
	{
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < s.length(); i++)
		{
			char ch = s.charAt(i);
			if (ch < '\u0080')
				sb.append(ch);
			else
			{
				String uc = String.format("%x", (int) ch);
				switch (uc.length())  // there has got to be a better way to do this!
				{
					case 1:
						sb.append(String.format("\\u000%s", uc));
						break;
					case 2:
						sb.append(String.format("\\u00%s", uc));
						break;
					case 3:
						sb.append(String.format("\\u0%s", uc));
						break;
					case 4:
						sb.append(String.format("\\u%s", uc));
						break;
				}
			}
		}

		return sb.toString();
	}

	static public FSM fromXML(String fileName) throws Exception
	{
		InputStream is = new FileInputStream(fileName);
		return fromXML(is);
	}

	static public FSM fromXML(InputStream xmlFile) throws Exception
	{
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		Document d = db.parse(xmlFile);
		XPath xpath = XPathFactory.newInstance().newXPath();
		Node node = null;

		node = (Node) xpath.evaluate("/fsm/@start", d, XPathConstants.NODE);
		String start = node.getNodeValue();

		String inputEncoding = "UTF-8";
		node = (Node) xpath.evaluate("/fsm/@inputDecoding", d, XPathConstants.NODE);
		if (node != null)
		{
			inputEncoding = node.getNodeValue();
		}

		String outputEncoding = "UTF-8";
		node = (Node) xpath.evaluate("/fsm/@outputEncoding", d, XPathConstants.NODE);
		if (node != null)
		{
			outputEncoding = node.getNodeValue();
		}

		// TODO: XPath expression should verify that <in> exists and contains a non empty string
		NodeList list = (NodeList) xpath.evaluate("//e", d, XPathConstants.NODESET);
		int len = list.getLength();
		FSMEntry[] fe = new FSMEntry[len];
		for (int i = 0; i < len; i++)
		{
			node = list.item(i);
			if (node.getNodeName().equals("e"))
			{
				String ss = ((Node) xpath.evaluate("s", node, XPathConstants.NODE)).getTextContent();
				String in = ((Node) xpath.evaluate("in", node, XPathConstants.NODE)).getTextContent();
				if (in == null || in.length() == 0)
				{
					continue;	// probably this xml was created automatically as an inverse of another xml file.
				}
				String out = ((Node) xpath.evaluate("out", node, XPathConstants.NODE)).getTextContent();
				Node next = ((Node) xpath.evaluate("next", node, XPathConstants.NODE));
				String[] s = ss.split(",");
				String regex = null;

				// check for \\u
				if (in.indexOf("\\u") != -1)
				{
					in = fixup(in);
				}
				if (out.indexOf("\\u") != -1)
				{
					out = fixup(out);
				}

				int idx = in.indexOf("\\");
				if (idx > 0)
				{
					regex = in.substring(idx + 1);
					in = in.substring(0, idx);
				}
				fe[i] = new FSMEntry(s, in, regex, out, next == null ? s[0] : next.getTextContent());
			}
		}

		return new FSM(start, inputEncoding, outputEncoding, fe);

	}

	private static String fixup(String s)
	{
		StringBuilder sb = new StringBuilder();
		int i = 0;
		while (i < s.length())
		{
			Pattern p = Pattern.compile("^[\\\\]u([0-9a-fA-F]{4})");
			Matcher m = p.matcher(s.substring(i));
			if (m.find())
			{
				String hex = m.group(1);
				sb.append((char) Integer.parseInt(hex, 16));
				i += 6;
			} else
			{
				sb.append(s.charAt(i));
				i++;
			}
		}

		return sb.toString();
	}

	private String Test(String line, int n, FSMEntry fe)
	{
		String match = "";
		boolean b = true;
		int lenfe = fe.edge.length();
		int lensb = line.length();
		int j = n;
		int k = 0;
		while (j < lensb && k < lenfe && b)
		{
			b = line.charAt(j++) == fe.edge.charAt(k++);
		}

		if (k == lenfe && b)
		{
			match = fe.edge;
			if (fe.regex != null)
			{
				int idx = n + match.length();
				Pattern p = Pattern.compile(fe.regex);
				try
				{
					Matcher m = p.matcher(line.substring(idx));
					boolean found = m.find();
					if (!found)
					{
						match = "";
					}
				} catch (Throwable ex)
				{
					System.out.println(ex);
				}
			}
		}
		return match;
	}

	public static String transform(String inputFile, boolean isFileName, String outputFile, String eol, FSM... fsm)
			  throws Exception
	{
		String[] lines = null;
		processLineNum = 0;
		BufferedReader reader = null;
		if (isFileName)
		{
			BufferedInputStream bis = null;
			InputStreamReader isr = null;
			FileInputStream fis = new FileInputStream(inputFile);
			bis = new BufferedInputStream(fis);
			if (fsm[0].inputDecoding.equals("raw"))
			{
				isr = new InputStreamReader(bis, new RawDecoder(Charset.forName("UTF-8"), 1F, 1F));
			} else
			{
				isr = new InputStreamReader(bis, Charset.forName(fsm[0].inputDecoding));
			}
			reader = new BufferedReader(isr);
		} else
		{
			lines = inputFile.split("\n");
		}



		BufferedOutputStream bos = null;
		OutputStreamWriter osw = null;
		ByteArrayOutputStream baos = null;
		if (outputFile != null)
		{
			FileOutputStream fos = new FileOutputStream(outputFile);
			bos = new BufferedOutputStream(fos);
		} else
		{
			baos = new ByteArrayOutputStream();
			bos = new BufferedOutputStream(baos);
		}

		int lastfsm = fsm.length - 1;
		while (fsm[lastfsm] == null)
		{
			lastfsm--;
		}

		if (fsm[lastfsm].outputEncoding.equals("raw"))
		{
			osw = new OutputStreamWriter(bos, new RawEncoder(Charset.forName("UTF-8"), 1F, 1F));
		} else
		{
			osw = new OutputStreamWriter(bos, Charset.forName(fsm[lastfsm].outputEncoding));
		}

		BufferedWriter writer = new BufferedWriter(osw);
		String line = null;
		int j = 0;
		while (true)
		{
			if (isFileName)
			{
				line = reader.readLine();
				if (line == null)
				{
					break;
				}
			} else
			{
				if (j >= lines.length)
				{
					break;
				} else
				{
					line = lines[j++];
				}
			}
			for (int i = 0; i < fsm.length; i++)
			{
				if (fsm[i] != null)
				{
					line = fsm[i].processLine(line);
				}
			}
			writer.write(line);
			writer.write(eol);
		}

		if (reader != null)
		{
			reader.close();
		}
		String result = null;
		writer.flush();

		if (baos != null)
		{
			result = baos.toString("UTF-8");
		}

		writer.close();
		return result;
	}

	static public byte[] toBytes(String s)
	{
		byte[] b = new byte[s.length()];
		for (int i = 0; i < b.length; i++)
		{
			b[i] = (byte) s.charAt(i);
		}
		return b;
	}

	static public String toUTF8(String s)
	{
		try
		{
			byte[] b = toBytes(s);
			return new String(b, "utf-8");
		} catch (Exception ex)
		{
			// since utf-8 is hard-coded, it is very likely we will ever reach here
			return s;
		}
	}

	static public String transformString(String s, FSM... fsm) throws Exception
	{
		String result = null;
		try
		{
			if (fsm[0] != null)
				result = FSM.transform(s, false, null, "", fsm);
			else
				result = s;
		} catch (Exception ex)
		{
			throw ex;
		}

		return result;
	}

	static public String[] createFileNames(String prefix, String source, String target)

		{
		String fileName[] = null;

		if (source.equals("slp1"))
		{
			fileName = new String[1];
			fileName[0] = String.format("%sslp1_%s.xml", prefix, target);
		} else if (target.equals("slp1"))
		{
			fileName = new String[1];
			fileName[0] = String.format("%s%s_slp1.xml", prefix, source);
		} else
		{
			fileName = new String[2];
			fileName[0] = String.format("%s%s_slp1.xml", prefix, source);
			fileName[1] = String.format("%sslp1_%s.xml", prefix, target);
		}

		return fileName;
	}

}
