package org.sanskritlibrary.web;

import java.io.IOException;
import java.util.ArrayList;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.sanskritlibrary.servlet.SanskritLibraryServlet;
import org.sanskritlibrary.Regex;
import java.util.Hashtable;
import org.sanskritlibrary.DictionaryLinks;
import org.sanskritlibrary.TextFormatter;

public class LexicalSources extends SanskritLibraryServlet {
//Note: the following is not used any more. This information is in data/miscellaneous/dictionary.xml
    private static String[] src = new String[]{
        "MW",
        "GetWord?key=deva&keyboard=yes&inputType=phonetic&unicodeInput=devInscript&phoneticInput=slp1&serverOptions=phonetic&viewAs=phonetic",
        //"http://www.sanskrit-lexicon.uni-koeln.de/monier/indexcaller.php?key=<SLP1>&filter=<filter>&translit=<translit>",
        "B",
        "http://www.sanskrit-lexicon.uni-koeln.de/scans/PWScan/disp1/index.php?key=<SLP1>&filter=<filter>&translit=<translit>",
        "PW",
        "http://www.sanskrit-lexicon.uni-koeln.de/scans/PWGScan/disp1/index.php?key=<SLP1>&filter=<filter>&translit=<translit>",
        "A",
        "http://dsal.uchicago.edu/cgi-bin/philologic/search3advanced?dbname=apte3&query=<roman>&matchtype=exact&display=utf8",
        "M",
        "http://dsal.uchicago.edu/cgi-bin/romadict.pl?query=<deva>&searchhws=yes&display=utf8&table=macdonell",
        "T",
        "http://dsal.uchicago.edu/cgi-bin/philologic/search3advanced?dbname=soas&query=<roman>&searchdomain=headwords&matchtype=exact&display=utf8",
        "R",
        "{location}/Ramopakhyana?key=<SLP1>&filter=<filter>",
        "pw",
        "{location}/NotImplemented?type=pw&name=The+small+St.+Petersburg+lexicon+of+Bohtlingk+and+Roth",
        "SKD",
        "{location}/NotImplemented?type=SKD&name=Sabdakalpadruma",
        "L",
        "{location}/NotImplemented?type=L&name=Indigenous+Indian+lexical+sources",
        "V",
        "{location}/NotImplemented?type=V&name=Vacaspatyam",
        "W",
        "{location}/NotImplemented?type=W&name=Wilson",
        "Huet",
        "http://sanskrit.inria.fr/cgi-bin/sktindex?t=KH&q=<hk>"
    };
    private static Hashtable<String, String> h = new Hashtable<String, String>();

    static {
        for (int i = 0; i < src.length - 1; i++) {
            h.put(src[i], src[i + 1]);
        }
    }

    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        
        String dictionaryType = request.getParameter("dictionaryType");
        String key = getParameterValue(request, "key", "");
        String orig = getParameterValue(request, "orig", key);
        String filter = request.getParameter("filter");
        if (filter == null) {
            filter = convertFilter(request);
        }
        //String filter=convertFilter(request);
        String translit = getParameterValue(request, "translit", "SLP");
        String currentDomain = this.getServletContext().getInitParameter("currentDomain");
        String appName = this.getServletContext().getInitParameter("appName");
        String url = String.format("http://%s/%s", currentDomain, appName);

        if (dictionaryType != null) {
            buildDictionaryLink(request, response, dictionaryType, key, filter, translit, url);
        } else {
            buildIFrameLinks(request, response, key, orig, filter, translit, url);
        }
    }

    // build a link suitable for call a dictionary of type -dictionaryType- with the given word.
    private void buildDictionaryLink(HttpServletRequest request, HttpServletResponse response,
            String dictionaryType, String key, String filter, String translit, String url) throws IOException, ServletException {
        String link = buildLink(dictionaryType, key, filter, translit, url);
        this.writeResult(response, link);
    }

    // Convert encoding type of server display to filter
    private String convertFilter(HttpServletRequest request) {
        String filter = this.getCookieValue(request, "serverOptions");
        String filterx = "SktDevaUnicode"; // default value

        if (filter.equals("deva")) {
            filterx = "SktDevaUnicode";
        } else if (filter.equals("roman")) {
            filterx = "SktRomanUnicode";
        } else if (filter.equals("phonetic")) {
            String phoneticInput = getCookieValue(request, "phoneticInput");
            if (phoneticInput.equals("hk")) {
                filterx = "SLP2HK";
            } else if (phoneticInput.equals("slp1")) {
                filterx = "SLP2SLP";
            } else if (phoneticInput.equals("it")) {
                filterx = "SLP2ITRANS";
            }
        }

        return filterx;
    }

    // This is old way of doing it using iframes, one for each dictionary. This was a bandwidth hog because all dictionary
    // entries were fetched whether the user looked at them or not.
    private void buildIFrameLinks(HttpServletRequest request, HttpServletResponse response,
            String key, String orig, String filter, String translit, String url) throws IOException, ServletException {
        TabInfo[] ti = null;
        FrameInfo[] fi = null;
        String tns = getCookieValue(request, "tabNames", "Monier-Williams");
        if (request.getParameter("tabNames") != null) {
            tns = request.getParameter("tabNames");
        }

        if (tns != null) // should alway be at least "MW"
        {
            String tabNames[] = tns.split("!");
            ti = new TabInfo[tabNames.length];
            fi = new FrameInfo[tabNames.length];

            int i = 0;

            for (String tabName : tabNames) {
                ti[i] = new TabInfo(i == 0 ? "tabSelected" : "tabUnselected", "tab" + (i + 1), tabName);
                String s = buildLink(tabName, key, filter, translit, url);
                fi[i] = new FrameInfo(i == 0 ? "frameSelected" : "frameUnselected", "frame" + (i + 1), tabName, s);
                i++;
            }
        } else {
            ti = new TabInfo[0];
            fi = new FrameInfo[0];
        }

        request.setAttribute("tabs", ti);
        request.setAttribute("dict", tns);
        request.setAttribute("frames", fi);
        request.setAttribute("word", key);
        request.setAttribute("orig", orig);
        request.setAttribute("context", " <a href=\"Reference\">Reference</a> | ");
        request.setAttribute("title", "Integrated Sanskrit dictionary");
        ArrayList<DictionaryLinks.DictionaryLink> links = ((DictionaryLinks) getServletContext().getAttribute("dictionaryLinks")).getLinks();
            request.setAttribute("dictionaryLinks", links);
        RequestDispatcher rd = request.getRequestDispatcher("LexicalSources.jsp");
        rd.forward(request, response);
    }

    private String buildLink(String tabName, String key, String filter, String translit, String url) {
        DictionaryLinks dl = (DictionaryLinks) this.getServletContext().getAttribute("dictionaryLinks");
        String s = dl.getLink(tabName).pattern;
        if (s != null) {
            if (s.contains("{SLP1}")) {
                s = Regex.replace(s, "\\{SLP1\\}", key);
                s = Regex.replace(s, "\\{filter\\}", filter);
                s = Regex.replace(s, "\\{translit\\}", translit);
            } else if (s.contains("{roman}")) {
                String roman = transformString(key, "*:SLP1->roman");
                if (roman == null) {
                    roman = this.getError();
                } else {
                    roman = TextFormatter.charToEscapedUTF8(roman);
                }
                s = Regex.replace(s, "\\{roman\\}", roman);
            } else if (s.contains("{deva}")) {
                String deva = transformString(key, "*:SLP1->deva");
                if (deva == null) {
                    deva = this.getError();
                } else {
                    deva = TextFormatter.charToEscapedUTF8(deva);
                }
                s = Regex.replace(s, "\\{deva\\}", deva);
            } else if (s.contains("{hk}")) {
                String hk = transformString(key, "*:SLP1->hk");
                if (hk == null) {
                    hk = this.getError();
                } else {
                    hk = TextFormatter.charToEscapedUTF8(hk);
                }
                s = Regex.replace(s, "\\{hk\\}", hk);
            }
        } else {
            s = String.format("{location}/NotImplemented?type=%s&name=Unknown+source+type", tabName);
        }

        s = s.replace("{location}", url);       // for stuff that is on Sanskrit Library server
        return s;
    }

    @Override
    public void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
    }

    /**
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
}
