package org.sanskritlibrary.web;

import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletOutputStream;
import java.util.Hashtable;
import org.sanskritlibrary.webservice.UniversalProxy;
import org.sanskritlibrary.webservice.ByteBuffer;
import org.sanskritlibrary.servlet.SanskritLibraryServlet;

/**
 *
 * @author rbunker
 */
public class MorphologicalAnalyzer extends SanskritLibraryServlet {
    //http://archimedes.fas.harvard.edu/cgi-bin/analyze

    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        request.setAttribute("context", " <a href=\"Tools\">Tools</a> | ");
        request.setAttribute("title", "Morphological Analyzer");

        String wordform = request.getParameter("wordform");
        if (wordform != null) {
            ServletOutputStream out = response.getOutputStream();
            UniversalProxy proxy = new UniversalProxy();
            Hashtable<String, String> h = createHashtable(request);
            h.put("Content-Type", "application/x-www-form-urlencoded; charset=UTF-8");
            wordform = convertToSLP1(null, getViewingType(request), wordform);
            String contentFormat = "wordform=%s&.submit=Submit+Query";
            String url = "http://archimedes.fas.harvard.edu/cgi-bin/analyze";
            String content = String.format(contentFormat, wordform);
            h.put("Content-Length", Integer.toString(content.length()));
            if (proxy.getResponse("POST", url, h, content.getBytes("UTF-8"))) {
                try {
                    proxy.getHeaders();
                    response.setContentType("text/html; charset=UTF-8");
                    ByteBuffer bb = proxy.getContent();
                    String result = new String(bb.content, 0, bb.len, "UTF-8");
                    String target = getCookieValue(request, "serverOptions", "deva");

                    if (target.equals("phonetic")) {
                        target = getCookieValue(request, "phoneticInput");
                    }
                    String map = "*:deva->" + target;
                    int idx = result.indexOf("<table");

                    if (idx != -1) // was a table returned?
                    {
                        result = result.substring(idx);  // only return the table
                        if (!target.equals("deva")) {
                            result = transformUnicodeString(result, map, 0x0900, 0x0980);
                            out.write(result.toString().getBytes("UTF-8"));
                        } else {
                            out.write(result.getBytes("UTF-8"));  // already deva
                        }
                    } else // return everything (probably an error message) but convert to target
                    {
                        idx = result.indexOf("Not found");
                        if (idx != -1) {
                            result = transformUnicodeString(result.substring(idx), map, 0x0900, 0x0980);
                        }
                        out.write(result.getBytes("UTF-8"));  // no table, probably illegal word form
                    }
                } catch (Exception ex) {
                    out.print(ex.getMessage());
                }
            } else {
                out.print(proxy.getError());
            }

            out.flush();
        } else {
            request.setAttribute("helpfile", "helpNotAvailable.html");
            RequestDispatcher rd = request.getRequestDispatcher("MorphologicalAnalyzer.jsp");
            rd.forward(request, response);
        }
    }

    /** 
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
}
