/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.sanskritlibrary.web;

import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.RequestDispatcher;
import org.sanskritlibrary.servlet.SanskritLibraryServlet;
import java.util.Hashtable;
import java.io.FileInputStream;
import java.io.ObjectInputStream;

// Looks up words in Peter's ramopakhyana.
public class Ramopakhyana extends SanskritLibraryServlet {

    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        String map = null;
        String filter = request.getParameter("filter");
        if (filter.equals("SktDevaUnicode")) {
            map = "*:slp1->deva";
        } else if (filter.equals("SktRomanUnicode")) {
            map = "*:slp1->roman";
        } else if (filter.equals("SLP2HK")) {
            map = "*:slp1->hk";
        } else if (filter.equals("SLP2ITRANS")) {
            map = "*:slp1->itrans";
        }

        String key = request.getParameter("key");
        try {
            String result = getEntry(key);
            if (result == null) {
                result = getEntry("%" + key);
            }
            if (result != null) {
                StringBuilder sb = new StringBuilder();
                String[] lines = result.split("\n");
                for (String line : lines) {
                    String[] fields = line.split("\t");
                    String wordnum = getField(fields, 1, ".0");
                    if (!wordnum.contains(".0")) {
                        // indent 1 tab for each dot in wordnum
                        int numtabs = 0;
                        for (int i = 0; i < wordnum.length(); i++) {
                            if (wordnum.charAt(i) == '.') {
                                numtabs++;
                            }
                        }
                        String headword = getField(fields, 4, "??");
                        String lexid = getField(fields, 5, "");
                        String meaning = getField(fields, 6, "");
                        if (map != null) {
                            headword = transformString(headword, map);
                        }
                        for (int i = 0; i < numtabs; i++) {
                            sb.append("\t");
                        }
                        sb.append(String.format("%s\t%s\t%s\n", headword, lexid, meaning));
                    }
                }
                request.setAttribute("result", sb.toString());
            } else {
                request.setAttribute("result", "Not found");
            }
        } catch (Exception ex) {
            request.setAttribute("result", String.format("System error (%s)", ex.getMessage()));
        }
        RequestDispatcher rd = request.getRequestDispatcher("Ramopakhyana.jsp");
        rd.forward(request, response);
    }

    String getField(String[] fields, int n, String dflt) {
        if (n < fields.length) {
            return fields[n];
        } else {
            return dflt;
        }
    }

    // Looks up key in hashtable of all words in the Ramopakhyana.
    String getEntry(String key) throws Exception {
        Hashtable<String, String> h = (Hashtable<String, String>) this.getServletContext().getAttribute("lexicon");
        if (h == null) {
            String datadir = this.getServletContext().getInitParameter("dataDirectory");
            //String lexiconPath = this.getServletContext().getInitParameter("dataDirectory") + "/lexicons/rAmopAKyAnaHead.obj";
            String lexiconPath = Utils.buildFileName(datadir, "lexicons/rAmopAKyAnaHead.obj");
            FileInputStream fis = new FileInputStream(lexiconPath);
            ObjectInputStream iis = new ObjectInputStream(fis);
            h = (Hashtable<String, String>) iis.readObject();
            this.getServletContext().setAttribute("lexicon", h);
        }

        return h.get(key);
    }

    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
}
