package org.sanskritlibrary.web;

public class SLTexts
{
    // Format
    public static final int CONTINUOUS = 1;
    public static final int SANDHI = 2;
    public static final int CONTINUOUS_SANDHI = 4;
    public static final int READER = 8;
    public static final int RECONSTRUCTED = 16;

    // Period
    public static final int VEDIC = 32;
    public static final int CLASSICAL = 64;
    public static final int EPIC = 128;
    public static final int ANY_PERIOD = VEDIC | CLASSICAL | EPIC;

    // Source
    public static final int TITUS = 256;
    public static final int CARDONA = 512;
    public static final int SCHARF = 1024;

    static OptionValue[] sourceOptions =
    {
        new OptionValue("all", "All"),
        new OptionValue("titus", "TITUS"),
        new OptionValue("cardona", "Cardona"),
        new OptionValue("scharf", "Scharf")
    };

    static OptionValue[] periodOptions =
    {
        new OptionValue("all", "All"),
        new OptionValue("vedic", "Vedic"),
        new OptionValue("epic", "Epic"),
        new OptionValue("classical", "Classical")
    };

    static OptionValue[] formatOptions =
    {
        new OptionValue("all", "All"),
        new OptionValue("continuous", "Continuous"),
        new OptionValue("sandhi", "Sandhi-Analyzed"),
        new OptionValue("wordsep", "Word-Separated"),
        new OptionValue("both", "Both"),
        new OptionValue("reader", "Reader")
    };

    static SLText[] sltexts = new SLText[]
    {
        new SLText("avs", "Atharva-Veda-Saṃhitā: Śaunaka recension",TITUS,VEDIC,CONTINUOUS),
        new SLText("amrtabup", "Amṛtabindu-Upaniṣad",TITUS,VEDIC,CONTINUOUS),
        new SLText("kautarth", "Arthaśāstra of Kauṭilya",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("vagbhahs", "Aṣṭāṅgahṛdayasaṃhitā of Vāgbhaṭa",TITUS,CLASSICAL,CONTINUOUS),
        new SLText("azwADyAyI", "A\u1e63\u1e6d\u0101dhy\u0101y\u012b", CARDONA,CLASSICAL,READER),
        new SLText("agngs", "Āgniveśya-Gṛhyasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("atmaup", "Ātma-Upaniṣad",TITUS,VEDIC,CONTINUOUS),
        new SLText("apgs", "Āpastamba-Gṛhyasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("apdhs", "Āpastamba-Dharmasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("apsbs", "Āpastamba-Śulbasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("apss", "Āpastamba-Śrautasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("aryabhat", "Āryabhaṭīya of Brahmagupta",TITUS,CLASSICAL,SANDHI),
        new SLText("aruneyup", "Āruṇeyi-Upaniṣad",TITUS,VEDIC,CONTINUOUS),
        new SLText("isaup", "Īśa-Upaniṣad",TITUS,VEDIC,CONTINUOUS_SANDHI),
        new SLText("rvkh", "Ṛg-Veda Khilāni",TITUS,VEDIC,CONTINUOUS),
        new SLText("fgveda_prAtiSAKya", "\u1e5agveda pr\u0101ti\u015b\u0101khya", CARDONA, CLASSICAL, SANDHI),
        new SLText("rv1", "Ṛgveda-Saṃhitā",TITUS,VEDIC,RECONSTRUCTED),
        new SLText("rv10", "Ṛgveda-Saṃhitā",TITUS,VEDIC,RECONSTRUCTED),
        new SLText("rv2", "Ṛgveda-Saṃhitā",TITUS,VEDIC,RECONSTRUCTED),
        new SLText("rv3", "Ṛgveda-Saṃhitā: Book III",TITUS,VEDIC,RECONSTRUCTED),
        new SLText("rv4", "Ṛgveda-Saṃhitā: Book IV",TITUS,VEDIC,RECONSTRUCTED),
        new SLText("rv5", "Ṛgveda-Saṃhitā: Book V",TITUS,VEDIC,RECONSTRUCTED),
        new SLText("rv6", "Ṛgveda-Saṃhitā: Book VI",TITUS,VEDIC,RECONSTRUCTED),
        new SLText("rv7", "Ṛgveda-Saṃhitā: Book VII",TITUS,VEDIC,RECONSTRUCTED),
        new SLText("rv8", "Ṛgveda-Saṃhitā: Book VIII",TITUS,VEDIC,RECONSTRUCTED),
        new SLText("rv9", "Ṛgveda-Saṃhitā: Book IX",TITUS,VEDIC,RECONSTRUCTED),
        new SLText("rtusam", "Ṛtusaṃhāra of Kālsidāsa",TITUS,CLASSICAL,CONTINUOUS),
        new SLText("aitup", "Aitareya-Upaniṣad",TITUS,VEDIC,CONTINUOUS_SANDHI),
        new SLText("aitupjz", "Aitareya-Upaniṣad: as edited by Mislav Ježic",TITUS,VEDIC,CONTINUOUS),
        new SLText("ab", "Aitareya-Brāhmaṇa",TITUS,VEDIC,CONTINUOUS),
        new SLText("kathup", "Kāṭha-Upaniṣad",TITUS,VEDIC,CONTINUOUS_SANDHI),
        new SLText("kathgs", "Kāṭhaka-Gṛhyasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("kASikA", "K\u0101\u015bik\u0101", CARDONA, CLASSICAL, SANDHI),
        new SLText("katysbs", "Kāṭyāyana-Śulbasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("katys", "Kātyāyana-Smṛti: Sāra-Uddhāra",TITUS,VEDIC,CONTINUOUS),
        new SLText("kamasutr", "Kāmasūtra of Vatsyāyana",TITUS,CLASSICAL,CONTINUOUS_SANDHI),
        new SLText("kumara", "Kumārasaṃbhava of Kālidāsa",TITUS,CLASSICAL,CONTINUOUS_SANDHI),
        new SLText("kenup", "Kena-Upaniṣad",TITUS,VEDIC,CONTINUOUS_SANDHI),
        new SLText("kaivalup", "Kaivalya-Upaniṣad",TITUS,VEDIC,CONTINUOUS),
        new SLText("kauthgs", "Kauthuma-Gṛhyasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("kauss", "Kauśika-Sūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("kausup", "Kauśītaki-Upaniṣad",TITUS,VEDIC,CONTINUOUS),
        new SLText("kausgs", "Kauśītaki-Gṛhya-Sūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("khadgs", "Khādira-Gṛhyasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("gobhgs", "Gobhila-Gṛhyasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("gautdhs", "Gautamīya-Dharmasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("chup", "Chāndogya-Upaniṣad",TITUS,VEDIC,CONTINUOUS_SANDHI),
        new SLText("jub", "Jaiminīya-Upaniṣad-Brāhmaṇa",TITUS,VEDIC,CONTINUOUS),
        new SLText("jaigs", "Jaimini-Ghṛhyasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("tantrakh", "Tantrākhyāyika",TITUS,CLASSICAL,CONTINUOUS_SANDHI),
        new SLText("vartikn", "Tikanikayātrā of Varāhamihira",TITUS,CLASSICAL,CONTINUOUS),
        new SLText("ta1", "Taittirīya-Āraṇyaka Prap\u0101\u1e6daka 1",TITUS,VEDIC,CONTINUOUS),
        new SLText("ta4", "Taittirīya-Āraṇyaka Prap\u0101\u1e6daka 4",TITUS,VEDIC,CONTINUOUS),
        new SLText("ta5", "Taittirīya-Āraṇyaka Prap\u0101\u1e6daka 5",TITUS,VEDIC,CONTINUOUS),
        new SLText("tejabup", "Tejabindu-Upaniṣad",TITUS,VEDIC,CONTINUOUS),
        new SLText("taittup", "Taittirīya-Upaniṣad",TITUS,VEDIC,CONTINUOUS),
        new SLText("tp", "Taittirīya-Prātiśākhya",TITUS,VEDIC,CONTINUOUS),
        new SLText("taittriya_prAtiSAKya", "Taittriya pr\u0101ti\u015b\u0101khya", CARDONA, CLASSICAL, SANDHI),
        new SLText("tb", "Taittirīya-Brāhmaṇa",TITUS,VEDIC,CONTINUOUS),
        new SLText("ts", "Taittirīya-Saṃhitā",TITUS,VEDIC,CONTINUOUS),
        new SLText("dasakum", "Daśakumāracarita of Daṇḍin",TITUS,CLASSICAL,CONTINUOUS_SANDHI),
        new SLText("naraddhs", "Nārada-Smṛti",TITUS,VEDIC,CONTINUOUS),
        new SLText("niGaRwu", "Nigha\u1e47\u1e6du", CARDONA, CLASSICAL, SANDHI),
        new SLText("nirukta", "Nirukta", CARDONA, ANY_PERIOD, SANDHI),
        new SLText("pancatt", "Pañcatantra",TITUS,CLASSICAL,CONTINUOUS_SANDHI),
        new SLText("paYcatantra", "Pañcatantra",SCHARF,CLASSICAL,CONTINUOUS),
        new SLText("paramaup", "Paramahaṃsa-Upaniṣad",TITUS,VEDIC,CONTINUOUS),
        new SLText("parss", "Parāśara-Smṛti",TITUS,VEDIC,CONTINUOUS),
        new SLText("pargs", "Pāraskara-Gṛhyasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("baskmup", "Bāṣkalamantra-Upaniṣad",TITUS,VEDIC,CONTINUOUS),
        new SLText("varbrjat", "Bṛhajjātaka of Varāhamihira",TITUS,CLASSICAL,SANDHI),
        new SLText("brhats", "Bṛhatsaṃhitā of Varāhamihira",TITUS,CLASSICAL,CONTINUOUS),
        new SLText("bps", "Bṛhaspati-Smṛti",TITUS,VEDIC,CONTINUOUS),
        new SLText("bau", "Bṛhadāraṇyaka-Upaniṣad: Mādhyandina-Recension / Kāṇva-Recension",TITUS,VEDIC,CONTINUOUS_SANDHI),
        new SLText("baudhgs", "Baudhāyana-Gṛhyasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("baudhdhs", "Baudhayana-Dharmasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("baudhsbs", "Baudhayana-Śulbasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("baudhss", "Baudhāyana-Śrautasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("brahmaup", "Brahma-Upaniṣad",TITUS,VEDIC,CONTINUOUS),
        new SLText("brahms", "Brahmasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("brsphsd", "Brāhmasphuthasiddhānta of Brahmagupta",TITUS,CLASSICAL,SANDHI),
        new SLText("bhargs", "Bhāradvāja-Gṛhyasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("bhaskbij", "Bhījagaṇita of Bhāskara",TITUS,CLASSICAL,SANDHI),
        new SLText("bhasklil", "Līlāvatī of bhāskara",TITUS,CLASSICAL,SANDHI),
        new SLText("manu", "Manu-Smṛti",TITUS,VEDIC,CONTINUOUS),
        new SLText("manb", "Mantra-Brāhmaṇa",TITUS,VEDIC,CONTINUOUS_SANDHI),
        new SLText("mandukya", "Māṇḍūkya-Upaniṣad",TITUS,VEDIC,CONTINUOUS_SANDHI),
        new SLText("mangs", "Mānava-Gṛhyasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("mansbs", "Mānava-Śulbasutra",TITUS,VEDIC,CONTINUOUS),
        new SLText("mbh01", "Mahābhārata: Ādiparvan",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("mbh02", "Mahābhārata: Sabhāparvan",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("mbh03", "Mahābhārata: Vanaparvan",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("mbh04", "Mahābhārata: Virāṭaparvan",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("mbh05", "Mahābhārata: Udyogaparvan",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("mbh06", "Mahābhārata: Bhīṣmaparvan",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("mbh07", "Mahābhārata: Droṇaparvan",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("mbh08", "Mahābhārata: Karṇaparvan",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("mbh09", "Mahābhārata: Śalyaparvan",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("mbh10", "Mahābhārata: Sauptikaparvan",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("mbh11", "Mahābhārata: Strīparvan",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("mbh12", "Mahābhārata: Śāntiparvan",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("mbh13", "Mahābhārata: Anuśāsanaparvan",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("mbh14", "Mahābhārata: Āśvamedhikaparvan",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("mbh15", "Mahābhārata: Āśramavāsikaparvan",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("mbh16", "Mahābhārata: Mausalaparvan",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("mbh17", "Mahābhārata: Mahāprasthānikaparvan",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("mbh18", "Mahābhārata: Svargārohaṇaparvan",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("mahABAzya", "Mah&#x0101;bh&#x0101;&#x1e63;ya", CARDONA, CLASSICAL, SANDHI),
        new SLText("mundaka", "Muṇḍaka-Upaniṣad",TITUS,VEDIC,CONTINUOUS_SANDHI),
        new SLText("meghadut", "Meghadūta of Kālidāsa",TITUS,CLASSICAL,CONTINUOUS),
        new SLText("ys", "Yājñavalkya-Smṛti",TITUS,VEDIC,CONTINUOUS),
        new SLText("varyogay", "Yogayātra of Varāhamihira",TITUS,CLASSICAL,CONTINUOUS),
        new SLText("ram01", "Rāmāyaṇa of Vālamīki: Bālakāṇḍa",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("ram02", "Rāmāyaṇa of Vālamīki: Ayodhyakāṇḍa",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("ram03", "Rāmāyaṇa of Vālamīki: Araṇyakāṇḍa",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("ram04", "Rāmāyaṇa of Vālamīki: Kiṣkindhākāṇḍa",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("ram05", "Rāmāyaṇa of Vālamīki: Mundarakāṇḍa",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("ram06", "Rāmāyaṇa of Vālamīki: Yuddhakāṇḍa",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("ram07", "Rāmāyaṇa of Vālamīki: Uttarakāṇḍa",TITUS,EPIC,CONTINUOUS_SANDHI),
        new SLText("rAmopAKyAna", "R&#x0101;mop&#x0101;khy&#x0101;na", SCHARF, EPIC, CONTINUOUS_SANDHI),
        new SLText("vb", "Vaṃśa-Brāhmaṇa",TITUS,VEDIC,CONTINUOUS),
        new SLText("vajasaneyi_prAtiSAKya", "Vajasaneyi pr\u0101ti\u015b\u0101khya", CARDONA, CLASSICAL, SANDHI),
        new SLText("vs", "Vājasaneyi-Saṃhitā: Mādhyandina-Recension",TITUS,VEDIC,CONTINUOUS),
        new SLText("vargs", "Vārāha-Gṛhyasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("vArttika", "V&#x0101;rttika", CARDONA, CLASSICAL, SANDHI),
        new SLText("vikrambr", "Vikramacarita: Brief Recension",TITUS,CLASSICAL,CONTINUOUS),
        new SLText("vikramjr", "Vikramacarita: Jainistic Recension",TITUS,CLASSICAL,CONTINUOUS),
        new SLText("vikrammr", "Vikramacarita: Metrical Recension",TITUS,CLASSICAL,CONTINUOUS),
        new SLText("vikramsr", "Vikramacarita: Southern Recension",TITUS,CLASSICAL,CONTINUOUS),
        new SLText("vikramvr", "Vikramacarita: Vararuci Recension",TITUS,CLASSICAL,CONTINUOUS),
        new SLText("varvivpt", "Vivāhapaṭala of Varāhamihira",TITUS,CLASSICAL,CONTINUOUS),
        new SLText("vetaljam", "Vetālapañcaviṃśati of Jambhalladatta",TITUS,CLASSICAL,CONTINUOUS_SANDHI),
        new SLText("vaikhdhs", "Vaikhānasa-Dharmasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("vaikhgs", "Vaikhānasa-Gṛhyasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("vaits", "Vaitāna-Sūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("visnudhs", "Vaiṣṇava-Dharmasūtra",TITUS,VEDIC,CONTINUOUS),
        new SLText("sbm", "Śatapatha-Brāhmaṇa: Mādhyandina-Recension",TITUS,VEDIC,CONTINUOUS),
        new SLText("svetup", "Śvetāśvatara-Upaniṣad",TITUS,VEDIC,CONTINUOUS),
        new SLText("sankhgs", "Sāṅkhyāyana-Gṛhya-Sūtra",TITUS,VEDIC,CONTINUOUS_SANDHI),
        new SLText("svk", "Sāmaveda-Saṃhitā: Kauthumaśākha",TITUS,VEDIC,CONTINUOUS),
        new SLText("saundlah", "Saundaryalaharī of Śaṅkara",TITUS,CLASSICAL,CONTINUOUS),
        new SLText("hitopad", "Hitopadeśa of Nārāyaṇa",TITUS,CLASSICAL,CONTINUOUS_SANDHI),
        new SLText("hirgs", "Hiraṇyakeśi-Gṛhyasūtra",TITUS,VEDIC,CONTINUOUS),
    };

    public static SLText getText(String id)
    {
        for (SLText sltext : sltexts)
        {
            if (sltext.id.equals(id))
                return sltext;
        }
        return null;
    }

    public static SLText[] getTexts(int source, int period, int format)
    {
        // count # of texts of this type
        int n = 0;
        for (SLText sltext : sltexts)
        {
            if (sltext.id.equals("paYcatantra"))
                sltext.id = "paYcatantra";
            if (isa(sltext.source, source) &&
                isa(sltext.period, period) &&
                isa(sltext.format, format))
                n++;
        }

        SLText[] a = new SLText[n];
        n = 0;
        for (SLText sltext : sltexts)
        {
            if (isa(sltext.source, source) &&
                isa(sltext.period, period) &&
                isa(sltext.format, format))
                a[n++] = sltext;
        }

        return a;
    }

    private static boolean isa(int type, int request)
    {
        return (type | request) == request;
    }

    public static OptionValue[] getSources()
    {
        return sourceOptions;
    }

    public static OptionValue[] getPeriods()
    {
        return periodOptions;
    }

    public static OptionValue[] getFormats()
    {
        return formatOptions;
    }
}
