package org.sanskritlibrary.web;

// See http://onjava.com/pub/a/onjava/2001/04/12/listeners.html

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import org.sanskritlibrary.SandhiHeuristics;
import org.sanskritlibrary.DictionaryLinks;

public class StartupListener implements ServletContextListener
{
    public void contextInitialized(ServletContextEvent sce)
    {
        String datadir = sce.getServletContext().getInitParameter("dataDirectory");
        String apacheDirectory = sce.getServletContext().getInitParameter("apacheDirectory");
        if (apacheDirectory == null)
            apacheDirectory = "";
        if (apacheDirectory.endsWith("/"))
            apacheDirectory = apacheDirectory.substring(0, apacheDirectory.length()-1);
        sce.getServletContext().setAttribute("apacheDirectory", apacheDirectory);

        String currentDomain = sce.getServletContext().getInitParameter("currentDomain");
        sce.getServletContext().setAttribute("currentDomain", currentDomain);

        String result = loadConfigFiles(sce.getServletContext());
        if (result.length() > 0)
            System.out.println(result);         // write to log file.
    }

    static public String loadConfigFiles(ServletContext sc)
    {
        StringBuilder sb = new StringBuilder();

        String datadir = sc.getInitParameter("dataDirectory");
        //String sandhiHeuristicsFile = sc.getInitParameter("sandhiHeuristicsFile");
        String sandhiHeuristicsFile = Utils.buildFileName(datadir, "miscellaneous/sandhiHeuristics.xml");
        try
        {
            if (sandhiHeuristicsFile != null)
            {
                SandhiHeuristics sh = new SandhiHeuristics(sandhiHeuristicsFile);
                sc.setAttribute("sandhiHeuristics", sh);
            }
            else
                sb.append("The file tomcat/conf/context.xml needs an entry\n<Parameter name='sandhiHeuristicsFile' value='...' override='false'/>\n");
        }
        catch (Exception ex)
        {
            sb.append(String.format("Error attempting to load sandhi heuristics from file %s:\nError: %s\n", sandhiHeuristicsFile, ex.getMessage()));
        }

        //String dictionaryLinksFile = sc.getInitParameter("dictionaryLinksFile");
        String dictionaryLinksFile = Utils.buildFileName(datadir, "miscellaneous/dictionaryLinks.xml");
        try
        {
            if (dictionaryLinksFile !=  null)
            {
                DictionaryLinks dl = new DictionaryLinks(dictionaryLinksFile);
                sc.setAttribute("dictionaryLinks", dl);
            }
            else
                sb.append("The file tomcat/conf/context.xml needs an entry\n<Parameter name='dictionaryLinksFile' value='...' override='false'/>\n");
        }
        catch (Exception ex)
        {
            sb.append(String.format("Error attempting to load dictionary links from file %s:\nError: %s\n", dictionaryLinksFile, ex.getMessage()));
        }

        return sb.toString();
    }

    public void contextDestroyed(ServletContextEvent sce)
    {
    }
}
