package org.sanskritlibrary.web;

import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.sanskritlibrary.servlet.SanskritLibraryServlet;

public class Texts extends SanskritLibraryServlet
{

    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException
    {
        String index = request.getParameter("index");
        String currentsource = request.getParameter("source");
        String currentperiod = request.getParameter("period");
        String currentformat = request.getParameter("format");
        if (index == null)              // want the source/period/format drop lists
        {
            request.setAttribute("context", "");
            request.setAttribute("title", "Texts");
            request.setAttribute("sourceOptions", SLTexts.getSources());
            request.setAttribute("periodOptions", SLTexts.getPeriods());
            request.setAttribute("formatOptions", SLTexts.getFormats());
            request.setAttribute("currentsource", currentsource);           // note: these hidden fields are used only by browser code, there are never used to send information to the server
            request.setAttribute("currentperiod", currentperiod);
            request.setAttribute("currentformat", currentformat);
            forward("Texts.jsp", request, response);
        } 
        else
        {
            int sourceType = 0;
            if (currentsource == null)
                currentsource = "all";
            if (currentperiod == null)
                currentperiod = "all";
            if (currentformat == null)
                    currentformat = "all";

            if (currentsource.equals("all"))
                sourceType |= SLTexts.CARDONA | SLTexts.READER | SLTexts.TITUS;
            else if (currentsource.equals("titus"))
                sourceType |= SLTexts.TITUS;
            else if (currentsource.equals("cardona"))
                sourceType |= SLTexts.CARDONA;
            else if (currentsource.equals("scharf"))
                sourceType |= SLTexts.SCHARF;

            int periodType = 0;
            if (currentperiod.equals("all"))
                periodType |= SLTexts.CLASSICAL | SLTexts.EPIC | SLTexts.VEDIC;
            else if (currentperiod.equals("classical"))
                periodType |= SLTexts.CLASSICAL;
            else if (currentperiod.equals("epic"))
                periodType |= SLTexts.EPIC;
            else if (currentperiod.equals("vedic"))
                periodType |= SLTexts.VEDIC;

            int formatType = 0;
            if (currentformat.equals("all"))
                formatType |= SLTexts.CONTINUOUS | SLTexts.SANDHI | SLTexts.CONTINUOUS_SANDHI | SLTexts.READER;
            else if (currentformat.equals("continuous"))
                formatType |= SLTexts.CONTINUOUS;
            else if (currentformat.equals("sandhi"))
                formatType |= SLTexts.SANDHI;
            else if (currentformat.equals("both"))
                formatType |= SLTexts.CONTINUOUS_SANDHI;
            else if (currentformat.equals("reader"))
                formatType |= SLTexts.READER;

            String texttype = String.format("%s/%s/%s", currentsource, currentperiod, currentformat);
            String context = String.format("<a href='Texts?source=%s&period=%s&format=%s'>Texts</a> | ",
                                                currentsource, currentperiod, currentformat);
            request.setAttribute("context", context);
            request.setAttribute("title", String.format("Index to %s %s %s", currentsource, currentperiod, currentformat));

            request.setAttribute("texttype", texttype);

            SLText[] a = SLTexts.getTexts(sourceType, periodType, formatType);
            request.setAttribute("texts", a);
            if (a.length == 0)
                request.setAttribute("message", "No texts match that type.");

            forward("TextIndex.jsp", request, response);
        }
    }


    /** 
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>

}
