package org.sanskritlibrary.web;

import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.charset.Charset;
import java.util.Hashtable;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import javax.servlet.ServletException;
import javax.servlet.ServletInputStream;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.sanskritlibrary.IOAssistance;
import org.sanskritlibrary.servlet.SanskritLibraryServlet;
import org.sanskritlibrary.servlet.FileFilter;
import org.json.JSONObject;
import org.json.JSONArray;
import org.json.JSONException;
import org.sanskritlibrary.TextSegment;
import org.sanskritlibrary.TranscodingRule;
import org.sanskritlibrary.webservice.TransformMap;

public class TranscodeFile extends SanskritLibraryServlet
{
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
	{
        String s;
        request.setAttribute("context", " <a href=\"Tools.jsp\">Tools</a> | ");
        request.setAttribute("title", "Advanced transcoding");
        if (request.getParameter("herzfeld") != null) {
            request.getSession().setAttribute("herzfeld", true);
        }
        /*
		request.setAttribute("display", "textarea");
         */
		forward("TranscodeFile.jsp", request, response);
    }

	@Override
	protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
	{
        Object puzzle = request.getSession().getAttribute("herzfeld");
        if (puzzle != null) {
            Hashtable<String, String>params = this.getParameters(request, true);
            String filename = params.get("uploadFile.filename");
            byte[] bfr = (byte[])request.getAttribute(filename);
            request.setAttribute("serfile", filename);
            filename = this.getServletContext().getRealPath(filename);
            FileOutputStream fos = new FileOutputStream(filename);
            fos.write(bfr);
            fos.close();
            forward("ThanksDave.jsp", request, response);
            return;
        }

        List<TranscodingRule> transcodingRules = new ArrayList<TranscodingRule>();

        request.setAttribute("context", " <a href=\"Tools.jsp\">Tools</a> | ");
        request.setAttribute("title", "Transcoding");

		Hashtable<String, String>params = this.getParameters(request);
        String text = null;
        String uploadedFilename = params.get("uploadFile.filename");
		if (uploadedFilename != null)
			text = params.get("uploadFile");
        else
            text = params.get("text");

        try
        {
            String jsonEncodingRules = getCookieValue(request, "transcodingRules");
            if (jsonEncodingRules != null)
            {
                jsonEncodingRules = jsonEncodingRules.replace("+", "%2B");
                jsonEncodingRules = URLDecoder.decode(jsonEncodingRules, "UTF-8");
                JSONArray ja = new JSONArray(jsonEncodingRules);
                int len = ja.length();
                for (int i=0; i<len; i++)
                {
                    JSONObject jo = ja.getJSONObject(i);
                    String startTag = jo.getString("s");
                    String endTag = jo.getString("e");
                    String sourceEncoding = jo.getString("a");
                    String targetEncoding = jo.getString("b");
                    String newStartTag = jo.getString("ss");
                    String newEndTag = jo.getString("ee");
                    String gn = jo.getString("re");
                    TransformMap tm = null;
                    if (!sourceEncoding.equals("asis") && !targetEncoding.equals("asis")) {
                        tm = getTransformMap(String.format("*:%s->%s", sourceEncoding, targetEncoding));
                    }

                    transcodingRules.add(new TranscodingRule(startTag, endTag, newStartTag, newEndTag, gn, tm ));
                }

                Hashtable<String, Object> h = new Hashtable<String, Object>();

                org.sanskritlibrary.TranscodeFile tf = new org.sanskritlibrary.TranscodeFile();
                Hashtable<String, String> options = new Hashtable<String, String>();        // TODO put options in here
                String regexName = null;            // fix this
                List<TextSegment> segments = tf.findSegments(h, transcodingRules, text, regexName);
                text = tf.doEncodings(h, text, segments, uploadedFilename, transcodingRules);
            }
        }
        catch (Exception ex) {
            System.out.println(ex.getMessage());
        }

        saveFile(request, response, uploadedFilename, text);
        String link = "<a target='dl' href='download'>Download result</a><br/>";
        if (request.getParameter("isAjax") != null)         // if it is an Ajax call
        {
            sendLink(response, link);
        }
        else
        {
            request.setAttribute("result", link);
            forward("TranscodeFile.jsp", request, response);
        }
	}

    public void sendLink(HttpServletResponse response, String link) throws IOException
    {
        response.setContentType("text/html");

        ServletOutputStream sos = response.getOutputStream();
        sos.write(link.getBytes("UTF-8"));
        sos.flush();
        sos.close();
    }



    private void saveFile(HttpServletRequest request, HttpServletResponse response, String uploadedFilename, String text) throws IOException
    {
        String uploadDirectory = getServletContext().getInitParameter("uploadDirectory");
        String fn = this.getCookieValue(request, "transcodedFile");

        int idx = uploadedFilename.lastIndexOf(".");
        String ext = uploadedFilename.substring(idx+1);
        if (ext == null) {
            ext = "txt";
        }

        String contentType = FileFilter.contentTypes.get(ext);
        if (contentType == null) {
            contentType = "text/plain";
        }

        response.setContentType(contentType);

        if (fn == null) {
            fn = String.format("%s.%s", UUID.randomUUID(), ext);
            response.addCookie(new Cookie("transcodedFile", fn));
        }    
        
        String outfile = String.format("%s/%s", uploadDirectory, fn);
		BufferedWriter bw = IOAssistance.openWriter(outfile, Charset.forName("UTF-8"));
		bw.write(text);
		bw.close();
    }

    /**
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>

}
