package org.sanskritlibrary.web;

import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.sanskritlibrary.servlet.SanskritLibraryServlet;
import javax.servlet.ServletOutputStream;
import org.sanskritlibrary.TextFormatter;
import org.sanskritlibrary.IOAssistance;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.util.Hashtable;
import java.util.ArrayList;
import java.util.Date;
import java.nio.charset.Charset;

public class TranscodeText extends SanskritLibraryServlet {

    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        request.setAttribute("context", " <a href=\"Tools\">Tools</a> | ");
        request.setAttribute("title", "Transcoding");

        request.setAttribute("display", "textarea");
        doForward("TranscodeText.jsp", request, response, null);
    }

    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        request.setAttribute("context", " <a href=\"Tools.jsp\">Tools</a> | ");
        request.setAttribute("title", "Transcoding");

        boolean ajaxCall = true;
        Hashtable<String, String> params = this.getParameters(request);
        String source = params.get("lstSourceEncoding");
        String target = params.get("lstTargetEncoding");
        String text = null;
        String textParam = params.get("rbInput");
        boolean doTest = textParam != null && textParam.equals("test");
        if (params.get("uploadFile.filename") != null) {
            text = params.get("uploadFile");
            ajaxCall = false;
        } else if (doTest) {
            String fileName = null;
            try {
                String dataDir = this.getServletContext().getInitParameter("dataDirectory");
                //fileName = String.format("%s/TestVedicIn.txt", dataDir);
                fileName = Utils.buildFileName(dataDir, "miscellaneous/TestVedicIn.txt");
                if (getTests(request, fileName, target)) {
                    text = expand(request);
                } else {
                    text = null;
                }
            } catch (Exception ex) {
                text = null;
            }
            ajaxCall = false;
        } else {
            text = params.get("txtInput");
        }

        ServletOutputStream out = null;

        if (ajaxCall) {
            out = response.getOutputStream();
        }

        String map = String.format("*:%s->%s", source, target);
        String result = null;
        if (text != null) {
            result = transformString(text, map);
            if (!source.equals("slp1")) // need to convert to slp1 for appending to file
            {
                map = String.format("*:%s->slp1;", source);
                text = transformString(text, map);
                if (getError() != null) {
                    text = null;
                }
            }
        } else if (doTest) {
            result = "Tests could not be generated";
        } else {
            result = "Problem with uploaded file";
        }


        response.setContentType("text/text; charset=UTF-8");
        if (result == null) {
            String s = String.format("Error: %s", getError());
            if (out != null) {
                out.print(s);
            } else {
                request.setAttribute("result", s);
            }
        } else if (params.get("chkHex") != null) {
            StringBuilder sb = new StringBuilder();
            int len = result.length();
            for (int i = 0; i < len; i++) {
                char ch = result.charAt(i);
                if (ch > 0x7F) {
                    sb.append(String.format("&#x%s;", TextFormatter.toHexString(ch)));
                } else {
                    sb.append((char) ch);
                }
            }
            if (out != null) {
                out.write(sb.toString().getBytes("UTF-8"));
            } else {
                result = sb.toString();
            }
        } else {
            if (out != null) {
                out.write(result.getBytes("UTF-8"));
            }
        }

        String datadir = getServletContext().getInitParameter("dataDirectory");
        //String uploadDirectory = getServletContext().getInitParameter("uploadDirectory");
        String uploadDirectory = Utils.buildFileName(datadir, "uploads");
        if (out != null) // if it was an Ajax call
        {
            out.flush();
            /*
            // append slp1 version result to file "appendText".
            if (text != null)
            IOAssistance.append(uploadDirectory, "textEntry", text, "\n\n");
            else
            request.setAttribute("result", String.format("alert('System error: %s');", getError()));
             */
        } else // otherwise it was a postback
        {
            if (result != null) {
                if (text != null) // at this point text is the SLP1 version of the input
                {
                    if (!doTest) {
                        Date now = new Date();
                        String fileName = String.format("%s_%s.txt",
                                fixup(params.get("uploadFile.filename")),
                                fixup(now.toString()));
                        String outfile = String.format("%s/%s", uploadDirectory, fileName);
                        BufferedWriter bw = IOAssistance.openWriter(outfile, Charset.forName("UTF-8"));
                        bw.write(text);	// save SLP1 version of text
                        bw.close();
                        request.setAttribute("result", String.format("alert('File saved with name: %s');", fileName));
                        request.setAttribute("display", "textarea");
                    }
                    if (doTest) {
                        result = generateTestResults(request, result, target);
                        request.setAttribute("display", "pre");
                    }
                    request.setAttribute("txtInput", result);
                } else {
                    request.setAttribute("result", String.format("alert('System error: %s');", getError()));
                }
            }

            doForward("TranscodeText.jsp", request, response, params);
        }
    }

    // reads in vowels to use and bases required by current target.
    // saves them as attributes to be used later.
    private boolean getTests(HttpServletRequest request, String fileName, String target) throws Exception {
        BufferedReader br = IOAssistance.openReader(fileName);
        String line = null;
        do {
            line = br.readLine();
            if (line.startsWith(target)) {
                request.setAttribute("vowels", line.substring(target.length() + 1).split("\\s"));
            }
        } while (line.charAt(0) != '*');

        if (request.getAttribute("vowels") == null) {
            return false;
        }

        ArrayList<String> bases = new ArrayList<String>();
        bases.add("");		// for vowel letters
        request.setAttribute("bases", bases);
        String base = null;
        while ((base = br.readLine()) != null) {
            String s = base.trim();
            if (s.length() > 0) {
                bases.add(s);
            }
        }

        br.close();

        return true;
    }

    // Generates a table from base entries
    private String expand(HttpServletRequest request) throws Exception {
        StringBuilder sb = new StringBuilder();

        String line = null;
        String[] vowels = (String[]) request.getAttribute("vowels");
        ArrayList<String> bases = (ArrayList<String>) request.getAttribute("bases");
        int n = 1;
        String delim = "";
        for (String base : bases) {
            for (String vowel : vowels) {
                sb.append(String.format("%s%s%s", delim, base, vowel.equals(".") ? "" : vowel));
                delim = "\t";
            }
            sb.append("\n");
            n++;
        }

        return sb.toString();
    }

    private String generateTestResults(HttpServletRequest request, String result, String target) {
        StringBuilder sb = new StringBuilder();
        String[] vowels = (String[]) request.getAttribute("vowels");
        ArrayList<String> bases = (ArrayList<String>) request.getAttribute("bases");
        sb.append(String.format("<h3>%s</h3>", target));

        sb.append("<table style='border-collapse:collapse; text-align:center; border-spacing:0px;'>\n");
        sb.append("<tr><th>#</th><th>base</th>");
        for (String vowel : vowels) {
            sb.append(String.format("<th>%s</th>", vowel));
        }
        sb.append("</tr>\n");

        String map = String.format("*:%s->slp1", target);		// to test round trip
        String[] lines = result.split("\n");
        int i = 0;
        for (String line : lines) {
            sb.append("<tr>");
            String base = bases.get(i);
            sb.append(String.format("<td>%d</td><td>%s</td>", i + 1, base));


            String[] fields = line.split("\\s+");
            int j = 0;
            for (String field : fields) {
                String vowel = vowels[j];
                if (vowel.equals(".")) // virama
                {
                    if (i == 0 && j == 0) // at virama on vowel line
                    {
                        sb.append("<td>N/A</td>");
                        j++;
                        continue;
                    }
                    vowel = "";
                }
                if (field.length() > 0) {
                    String slp1 = transformString(field, map);	// do this to be sure that reverse mapping is okay
                    if (slp1 == null) {
                        slp1 = null;
                    }
                    if (!slp1.equals(base + vowel)) {
                        sb.append(String.format("<td><a style='color:red' onclick='showSLP1(\"%s\")'>%s</a></td>", slp1, field));
                    } else {
                        sb.append(String.format("<td>%s</td>", field));
                    }
                    j++;
                }
            }
            sb.append("</tr>\n");
            i++;
        }

        sb.append("</table>\n");

        return sb.toString();
    }

    private String fixup(String s) {
        s = s.replace('/', '_');
        s = s.replace('\\', '_');
        s = s.replace(':', '_');

        return s;
    }

    private void doForward(String name, HttpServletRequest request,
            HttpServletResponse response, Hashtable<String, String> h)
            throws ServletException, IOException {
        forward(name, request, response, h, "rbInput", "lstSourceEncoding", "lstTargetEncoding", "chkHex");
    }

    /** 
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
}
