package org.sanskritlibrary.web;

import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.RequestDispatcher;
import org.sanskritlibrary.servlet.SanskritLibraryServlet;
import java.util.Hashtable;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import org.sanskritlibrary.Regex;

public class WordAnalysis extends SanskritLibraryServlet
{   
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
	 {
		String map = null;
		String filter = request.getParameter("filter");
		if (filter.equals("SktDevaUnicode"))
			map = "*:slp1->deva";
		else if (filter.equals("SktRomanUnicode"))
			map = "*:slp1->roman";
		else if (filter.equals("SLP2HK"))
			map = "*:slp1->hk";
		else if (filter.equals("SLP2ITRANS"))
			map = "*:slp1->itrans";

		String text = request.getParameter("text");
		request.setAttribute("text", text);
		String word = request.getParameter("word");
		try
		{
			String result = getEntry(word, text);
			String wordMapped = word;
			if (result == null)
			{
				result = getEntry("%" + word, text);
				if (result == null && word.charAt(word.length()-1) == '.')
					result = getEntry(word.substring(0, word.length()-1), text);
			}
			if (result != null)
			{
				StringBuilder sb = new StringBuilder();
				if (map != null)
					wordMapped = transformString(word, map);
				sb.append(String.format("<h4>%s</h4>", wordMapped));
				sb.append("<table border='1'");
				String[] lines = result.split("\n");
				for (String line : lines)
				{
					String[] fields = line.split("\t");
					String wordnum = getField(fields, 1, ".0");
					wordnum = Regex.replace(wordnum, "^\\d+", "1");
					if (!wordnum.equals("1.0"))
					{
						String morphid = getField(fields, 3, "??");
						String headword = getField(fields, 4, "??");
						String headwordMapped = headword;
						String lexid = getField(fields, 5, "");
						if (map != null)
							headwordMapped = transformString(headword, map);
						headword = Regex.replace(headword, "[-+%\\s]", "");
						headword = Regex.replace(headword, "\\(.*$", "");
						sb.append(String.format(
	"<tr><td>%s</td><td>%s</td><td><a onclick='id_info(\"%s\")'>%s</a></td><td>%s</td></tr>\n",
							  wordnum, morphid, headword, headwordMapped, lexid));
					}
				}
				sb.append("</table>");
				request.setAttribute("result", sb.toString());
			}
			else
				request.setAttribute("result", "Not found");
			response.setCharacterEncoding("UTF-8");
			response.setContentType("text/html");
		}
		catch (Exception ex)
		{
			request.setAttribute("result", "System error");
		}
		RequestDispatcher rd = request.getRequestDispatcher("WordAnalysis.jsp");
		rd.forward(request, response);
    }

	 String getField(String[] fields, int n, String dflt)
	 {
		if (n < fields.length)
			return fields[n];
		else
			return dflt;
	 }

	 String getEntry(String key, String text) throws Exception
	 {
		 String hashAttributeName = "lexicon/" + text;
		 Hashtable<String, String> h = (Hashtable<String, String>)this.getServletContext().getAttribute(hashAttributeName);
		 if (h == null)
		 {
			String inflected = String.format("/lexicons/%sInflected.obj", text);
			String lexiconPath = this.getServletContext().getInitParameter("dataDirectory") + inflected;
			FileInputStream fis = new FileInputStream(lexiconPath);
			ObjectInputStream iis = new ObjectInputStream(fis);
			h = (Hashtable<String, String>)iis.readObject();
			this.getServletContext().setAttribute(hashAttributeName, h);
		 }

		 return h.get(key);
	 }
    /** 
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>

}
