package org.sanskritlibrary.web;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.sanskritlibrary.DictionaryLinks;
import org.sanskritlibrary.Regex;
import org.sanskritlibrary.SandhiHeuristics;
import org.sanskritlibrary.servlet.SanskritLibraryServlet;
import org.sanskritlibrary.webservice.RESTfulWSCaller;

/**
 *
 * @author ralph
 */
public class multidict extends SanskritLibraryServlet
{
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException
    {
        String forwardTo = null;
        try
        {
            String ptype = request.getParameter("type");
            int type = -1;
            if (ptype != null && ptype.length() > 0)
                type = ptype.charAt(0) - '0';

            if (type==0 ||          // this was originally initiated by user clicking a word link in text. No bread crumbs will be displayed.
                type==3)            // this was originally initiated by user clicking tools link. Bread crumbs will be displayed.
            {
                String uri = (String)this.getServletContext().getInitParameter("fflexLocation");
                if (uri == null)        // should never happen
                    uri = "http://sktlib.ccv.brown.edu/Sanskrit/Funderburk/fflex/web/rp1/getWord.php";
                Hashtable h = new Hashtable();
                h.put("filterin", "SLP2SLP");
                String outputType = getOutputType(request);
                h.put("filter", convertOutputType(outputType));
                String word = request.getParameter("selectedForm");
                String encoding = request.getParameter("encoding");
                if (encoding == null || encoding.length() == 0)      // this came directly from clicking a word in text
                    word = convertToSLP1(null, outputType, word);
                
                request.setAttribute("instructions", "Modify the inflected form and click the <b>Parse</b> button or click on a highlighted head-word to change the selection, then click on dictionary link");

                h.put("word", word);
                request.setAttribute("selectedForm", word);
                request.setAttribute("encoding", "SLP1");
                String result = RESTfulWSCaller.callText(uri, h);
                boolean found = result.indexOf("<tr>") != -1;
                if (!found)           // if word not found, try sandhi heuristics
                {

                    SandhiHeuristics sh = (SandhiHeuristics) getServletContext().getAttribute("sandhiHeuristics");
                    ArrayList<String> alts = sh.getAlternates(word);
                    for (String alt : alts)
                    {
                        h.put("word", alt);
                        result = RESTfulWSCaller.callText(uri, h);
                        found = result.indexOf("<tr>") != -1;
                        if (found)       // this heuristic worked!
                        {
                            request.setAttribute("rewritten", String.format("%s rewritten as %s", word, alt));
                            break;
                        }
                    }
                }

                if (found)      // choose first word
                {
                    Regex re = new Regex(result, "id_info\\(\"(.*?)\"\\)");
                    if (re.find())
                        request.setAttribute("selectedHeadWord", re.getGroup(1));
                    result = result.replaceAll("class='id_info'", "style='background-color: #dbe4ed;cursor:pointer'");
                }
                else
                    result = "Not found. Try modifying the form and then click the Parse button<br/>";

                result = result.replaceAll("<disp1>", "");
                request.setAttribute("result", result);
                request.setAttribute("type", Integer.toString(type));
                if (type == 0)
                    forwardTo = "FunderburkInterfaceNoBreadCrumbs.jsp";
                else
                {
                    request.setAttribute("context", " <a href=\"Tools\">Tools</a> | ");
                    request.setAttribute("title", "Look up inflected form");
                    forwardTo = "FunderburkInterfaceBreadCrumbs.jsp";
                }
            }
            else if (type == 1)             // called from tools page, user wants to look up head-word in dictionary
            {
                request.setAttribute("type", "1");          // will never show inflected form
                request.setAttribute("instructions", "Type in a headword and click a dictionary link.");
                request.setAttribute("context", " <a href=\"Reference\">Reference</a> | ");
                request.setAttribute("title", "Integrated Sanskrit dictionary");
                request.setAttribute("visibleInflectedForm", "display:none");
                forwardTo = "FunderburkInterfaceBreadCrumbs.jsp";
            }
            ArrayList<DictionaryLinks.DictionaryLink> links = ((DictionaryLinks) getServletContext().getAttribute("dictionaryLinks")).getLinks();
            request.setAttribute("dictionaryLinks", links);
            this.forward(forwardTo, request, response);
        } catch (Exception ex)
        {
            response.setStatus(response.SC_OK);
            this.writeResult(response, ex.getMessage());
        }
    }


    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
    {
    }

    private String convertOutputType(String outputType)
    {
        String filterx = "SktDevaUnicode"; // default value

        if (outputType.equals("deva"))
            filterx = "SktDevaUnicode";
        else if (outputType.equals("roman"))
            filterx = "SktRomanUnicode";
        else if (outputType.equals("hk"))
			filterx = "SLP2HK";
		else if (outputType.equals("slp1"))
			filterx = "SLP2SLP";
		else if (outputType.equals("it"))
			filterx = "SLP2ITRANS";

        return filterx;
}
    /** 
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>

}
