<%@page contentType="text/html" pageEncoding="UTF-8"%>

<%@include file="WEB-INF/jspf/Head.jspf"%>
<%@include file="WEB-INF/jspf/BreadCrumbs.jspf"%>
<%@include file="WEB-INF/jspf/Menu.jspf"%>
<%@include file="WEB-INF/jspf/EnglishPage.jspf"%>

<h2>Major activities</h2>
<p>The International Digital Sanskrit Library Integration project created a globally distributed, internet-based digital library in Sanskrit from formerly independent projects.  The project integrated projects to create Sanskrit digital archives, digital lexica, and linguistic software; to establish text-encoding standards; to enhance ancient and medieval manuscript access; and to develop OCR technology, display software, and Unicode-compliant text-editing software for Devanāgarī script.  The resulting integrated information system enriches access to digital content in Sanskrit located worldwide and thus enables broad use of this material for research and education.  The ready accessibility of web-based materials is especially significant for less commonly taught languages such as Sanskrit.</p>
<p>The project standardized Sanskrit text-encoding, revised the Unicode Standard to include characters necessary for Indic cultural heritage, supplied truthed data for optical character recognition, prepared the major digital Sanskrit-English lexicon for integration with linguistic software, produced several other digital lexical resources, produced a full-form Sanskrit lexicon and morphological analyzer, and fostered international collaboration in the area of Sanskrit computational linguistics.  The Sanskrit Computational Linguistics Consortium, founded at the Second International Sanskrit Computational Linguistics Symposium held at Brown under the project, continues to culture progress in the development of OCR of Indic scripts, critical editing software, generative grammars, parsing software, semantic networks, machine translation, tagged corpora, and integrated Sanskrit library software.</p>
<h2>Results</h2>
<h3>Standardization of Sanskrit text-encoding</h3>
<p>Scharf and Hyman completed a comprehensive survey of linguistic and theoretical issues related to the encoding of Sanskrit language and Devanāgarī script entitled <i>Linguistic Issues in Encoding Sanskrit</i> published by the premier Indian Indological book publisher Motilal Banarsidass.  The <a href='http://www.unicode.org/charts/' target='_blank'>Unicode 5.2 Character Code Charts page</a> includes the Devanagari Extended and Vedic Extensions code charts under South Asian Scripts and the <a href='Unicode' target='_blank'>Sanskrit Library Vedic Unicode page</a> details the history of the proposal with links to relevant documents.</p>
<p>While investigating phonetic distinctions made in the linguistic treatises proper to various Vedic traditions and collecting Vedic passages that illustrate each distinction, Scharf found evidence of the correspondence of accentuation marks with surface pitch rather than with underlying pitch.  These findings support arguments previously put forward by Michael Witzel and George Cardona concerning linguistic variation in Vedic and the value of accentual details for the linguistic and general history of India.  Scharf presented a paper on the topic called, “Vedic accent: underlying versus surface,” at the Fouth International Vedic Workshop, 24-27 May 2007 in Austin, TX (see the abstract under publications) and another “L'accent védique dans les traités de phonétique, les manuscrits et la récitation,” at the École des hautes études en sciences sociales, Paris, 1 December 2008.</p>
<h3>Devanāgarī OCR</h3>
<p>Govindaraju and Kompalli produced a prototype stochastic recognition-driven OCR engine for Devanāgarī that incorporates novel methodologies and promises to serve as the basis for future research in OCR for complex scripts.  Scharf and Hyman contributed a chapter, “Enhancing Access to Primary Cultural Heritage Materials of India,” to the <i>Guide to OCR for Indic Scripts: Document Recognition and Retrieval</i>, edited by Venu Govindaraju and Srirangaraj Setlur, published by Springer, 2009.
<h3>Digital lexicography</h3>
<p>The Cologne Digital Sanskrit Lexicon site now hosts an enhanced interface to three dictionaries that was developed in collaboration with project personnel at Brown in the International Digital Sanskrit Library Integration project.  These dictionaries include, besides MW, the <i>Sanskrit-Wörterbuch in kürzerer Fassung</i> by Otto Böhtlingk with the <i>Nachträge</i> by Richard Schmidt, and Apte's English-Sanskrit dictionary.  Moreover, the whole collection of Cologne's 10 dictionaries is accessible in the form of page images linked to headwords by the technique Funderburk developed in collaboration with project personnel.  These lexical sources are all accessible directly on the site of the <a href='http://www.sanskrit-lexicon.uni-koeln.de/' target='_blank'>Cologne Digital Sanskrit Lexicon project</a> as well as from the <a href='ReferenceWorks' target='_blank'>Sanskrit Library Reference page</a>.</p>
<p>The <a href='http://www.sanskritlibrary.org/Sanskrit/Vyakarana/Dhatupatha/index2.html' target='_blank'>canonical indices of the <i>Mādhavīya Dhātuvṛtti</i></a> Scharf developed with the collaboration of Funderburk are also accessible from the <a href='ReferenceWorks' target='_blank'>Sanskrit Library Reference page</a>. The <a href='MorphologicalAnalyzer' target='_blank'>nominal form morphological analyzer</a> developed by Scharf and Hyman is accessible on the <a href='Tools' target='_blank'>Sanskrit Library Tools page</a> as is the general <a href='FunderburkInterface?type=2' target='_blank'>inflectional morphology analyzer</a> Scharf developed in collaboration with Funderburk.  The Sanskrit reference works in the list Scharf provided Crane for scanning by the Toronto University Library under the Perseus Project are now part of the <a href='http://archive.org/details/texts' target='_blank'>Open Content Alliance archive</a>.</p>
<h3>Generative grammar and phonology</h3>
<p>Scharf and Hyman gave presentations at the First International Sanskrit Computational Linguistics Symposium, 29-31 October 2007 in Paris.  Scharf presented a paper on modeling the procedures of linguistic description used in Paninian generative grammar entitled, “Modelling Pāṇinian Grammar,” and Hyman on the automated transformation of our XML implementation of Panini's phonetic rules into a finite state transducer, entitled “From Pāṇinian Sandhi to Finite State Calculus.”  Both papers appear in the <a href='http://www.springer.com/computer/ai/book/978-3-642-00154-3' target='_blank'>volume of papers of the first two Sanskrit computational linguistics symposia</a> edited by Huet, Kulkarni and Scharf.  Scharf's paper, “Levels in Pāṇini’s <i>Aṣṭādhyāyī</i>,” appears in the volume of papers of the <a href='http://www.springer.com/computer/ai/book/978-3-540-93884-2' target='_blank'>Third International Symposium</a> edited by Huet and Kulkarni.</p>
<p>Related to his work in generation of Sanskrit inflected forms to build the full-form lexicon is another paper by Scharf.  “Pāṇinian accounts of the class eight presents,” a paper presented at the 216th Meeting of the American Oriental Society, 17-20 March 2006, Seattle, Washington, appeared in <i>Journal of the American Oriental Society</i> 128.3 (2008).</p>
<h3>Sanskrit library integration</h3>
<p>The integration of texts with grammatical analysis and lexical sources, which was the premier goal of the International Digital Sanskrit Library Integration project is available on the <a href='Texts' target='_blank'>Texts page</a> of the Sanskrit Library website.</p>
<h2>Project personnel</h2>
<ul>
<li>Peter M. Scharf, Principal Investigator</li>
<li>Malcolm D. Hyman, Co-Principal Investigator</li>
<li>Ramaswamy Chandrashekar, Post-Doctoral Research Associate</li>
<li>Susan J. Moore, Post-Doctoral Research Associate</li>
</ul>
<h2>Grant details</h2>
<ul>
<li>period: 1 January 2006 -- 30 December 2009</li>
<li>funding agency: National Science Foundation, Division of Information and Intelligent Systems</li>
<li>funding: $247,350</li>
<li>location: Brown University</li>
</ul>

<%@include file="WEB-INF/jspf/BodyEnd.jspf"%>
